*DECK NSSFL4
      SUBROUTINE NSSFL4(NUN,NG,NX,NY,LL4F,LL4X,LL4Y,NMIX,NALB,EPSOUT,
     > MAXOUT,MAT,XX,YY,XXX,YYY,IDL,VOL,KN,IQFR,QFR,DIFF,SIGR,CHI,SIGF,
     > SCAT,BETA,FD,BNDTL,IMPX,KEFF,SAVG)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Flux calculation for the analytic nodal method in Cartesian 2D
* geometry using the nodal correction iteration strategy.
*
*Copyright:
* Copyright (C) 2022 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NUN     number of unknowns per energy group.
* NG      number of energy groups.
* NX      number of nodes in the X direction.
* NY      number of nodes in the Y direction.
* LL4F    number of nodal flux unknowns.
* LL4X    number of nodal X-directed net currents unknowns.
* LL4Y    number of nodal Y-directed net currents unknowns.
* NMIX    number of mixtures in the nodal calculation.
* NALB    number of physical albedos.
* EPSOUT  convergence epsilon for the power method.
* MAXOUT  maximum number of iterations for the power method.
* MAT     material mixtures.
* XX      mesh spacings in the X direction.
* YY      mesh spacings in the Y direction.
* XXX     Cartesian coordinates along the X axis.
* YYY     Cartesian coordinates along the Y axis.
* IDL     position of averaged fluxes in unknown vector.
* VOL     node volumes.
* KN      node-ordered interface net current unknown list.
* IQFR    boundary condition information.
* QFR     albedo function information.
* DIFF    diffusion coefficients
* SIGR    removal cross sections.
* CHI     fission spectra.
* SIGF    nu times fission cross section.
* SCAT    scattering cross section.
* BETA    albedos.
* FD      discontinuity factors.
* BNDTL   set to 'flat', 'linear' or 'quadratic'.
* IMPX    edition flag.
*
*Parameters: output
* KEFF    effective multiplication factor.
* SAVG    unknown array (neutron fluxes and currents).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NUN,NG,NX,NY,LL4F,LL4X,LL4Y,NMIX,NALB,MAXOUT,IMPX,
     1 MAT(NX*NY),IDL(NX*NY),KN(6,NX,NY),IQFR(6,NX,NY)
      REAL EPSOUT,XX(NX*NY),YY(NX*NY),XXX(NX+1),YYY(NY+1),VOL(NX*NY),
     1 QFR(6,NX*NY),DIFF(NMIX,NG),SIGR(NMIX,NG),CHI(NMIX,NG),
     2 SIGF(NMIX,NG),SCAT(NMIX,NG,NG),BETA(NALB,NG,NG),FD(NMIX,4,NG,NG),
     3 KEFF,SAVG(NUN,NG)
      CHARACTER(LEN=12) :: BNDTL
*----
*  LOCAL VARIABLES
*----
      INTEGER, PARAMETER :: NZ=1,NDIM=2
      REAL(KIND=8) :: DEPS,DKEFF
      REAL :: COEF(4),KEFF_OLD,XYZ(4)
*----
*  ALLOCATABLE ARRAYS
*----
      REAL(KIND=8), ALLOCATABLE, DIMENSION(:,:,:) :: Lx,Rx,Ly,Ry
      REAL, ALLOCATABLE, DIMENSION(:) :: WORK1,WORK2,WORK4,WORK5,ZZ
      REAL, ALLOCATABLE, DIMENSION(:,:) :: WORK3,AGGXY
      REAL, ALLOCATABLE, DIMENSION(:,:,:) :: QFR2,DRIFT
      REAL(KIND=8), ALLOCATABLE, DIMENSION(:) :: EVECT
      REAL(KIND=8), ALLOCATABLE, DIMENSION(:,:) :: A
*
      ALB(X)=0.5*(1.0-X)/(1.0+X)
*----
*  SCRATCH STORAGE ALLOCATION
*----
      NEL=NX*NY
      N=LL4F*NG
      ALLOCATE(QFR2(6,NEL,NG),ZZ(NEL))
      ALLOCATE(Lx(NG,8*NG,NEL),Rx(NG,8*NG,NEL))
      ALLOCATE(Ly(NG,8*NG,NEL),Ry(NG,8*NG,NEL))
      ALLOCATE(DRIFT(6,NEL,NG),WORK1(NG),WORK2(NG),WORK3(NG,NG),
     > WORK4(NG),WORK5(NG))
      ALLOCATE(AGGXY(LL4F,LL4F),A(N,2*N),EVECT(N))
*----
*  ALBEDO PROCESSING
*----
      QFR2(:6,:NEL,:NG)=0.0
      DO IG=1,NG
        DO IQW=1,4
          DO I=1,NX
            DO J=1,NY
              IEL=(J-1)*NX+I
              IALB=IQFR(IQW,I,J)
              IF(IALB > 0) THEN
                IF(IALB.GT.NALB) CALL XABORT('NSSFL4: BETA OVERFLOW.')
                QFR2(IQW,IEL,IG)=QFR(IQW,IEL)*ALB(BETA(IALB,IG,IG))
              ELSE
                QFR2(IQW,IEL,IG)=QFR(IQW,IEL)
              ENDIF
            ENDDO
          ENDDO
        ENDDO
      ENDDO
*
      DRIFT(:6,:NEL,:NG)=0.0
      EVECT(:N)=1.0D0
      KEFF=1.E0
      KEFF_OLD=0.
*----
*  NODAL CORRECTION LOOP
*----
      ZZ(:NEL)=1.0
      JTER=0
      SAVG(:NUN,:NG)=0.0
      DO WHILE(ABS(KEFF_OLD-KEFF) >= EPSOUT)
        JTER=JTER+1
        IF(IMPX.GT.0) THEN
          WRITE(6,'(36H NSSFL4: Nodal correction iteration=,I5)')
     >    JTER
        ENDIF
        IF(JTER > MAXOUT) CALL XABORT('NSSFL4: NODAL ITERATION FAILURE')
        !
        ! set coarse mesh finite difference matrix
        A(:N,:2*N)=0.D0
        IOF=1
        DO IG=1,NG
          CALL NSSMXYZ(LL4F,NDIM,NX,NY,NZ,NMIX,MAT,XX,YY,ZZ,IDL,VOL,
     >    IQFR,QFR2(1,1,IG),DIFF(1,IG),DRIFT(1,1,IG),SIGR(1,IG),AGGXY)
          A(IOF:IOF+LL4F-1,IOF:IOF+LL4F-1)=AGGXY(:LL4F,:LL4F)
          DO IEL=1,NEL
            IBM=MAT(IEL)
            IF(IBM <= 0) CYCLE
            IND=IDL(IEL)
            IF(IND == 0) CYCLE
            JOF=1
            DO JG=1,NG
              IF(IG /= JG) A(IOF+IND-1,JOF+IND-1)=-VOL(IEL)*
     >        SCAT(IBM,IG,JG) ! IG <-- JG
              A(IOF+IND-1,N+JOF+IND-1)=VOL(IEL)*CHI(IBM,IG)*
     >        SIGF(IBM,JG)
              JOF=JOF+LL4F
            ENDDO
          ENDDO
          IOF=IOF+LL4F
        ENDDO
        CALL ALSBD(N,N,A,IER,N)
        IF(IER /= 0) CALL XABORT('NSSFL4: SINGULAR MATRIX.')
        !
        ! CMFD power iteration (use double precision)
        KEFF_OLD=KEFF
        DEPS=EPSOUT
        CALL AL1EIGD(N,A(1,N+1),DEPS,MAXOUT,ITER,EVECT,DKEFF,IMPX)
        KEFF=REAL(DKEFF)
        IF(IMPX > 0) WRITE(6,10) JTER,KEFF,ITER
        IF(IMPX > 1) THEN
          WRITE(6,'(1X,A)') 'NSSFL4: EVECT='
          IOF=0
          DO IG=1,NG
            WRITE(6,'(1X,1P,14D12.4)') EVECT(IOF+1:IOF+LL4F)
            IOF=IOF+LL4F
          ENDDO
        ENDIF
        !
        ! begin construct SAVG
        IF(NUN /= 5*LL4F+LL4X+LL4Y) CALL XABORT('NSSFL4: INVALID NUN.')
        DO IEL=1,LL4F
          DO IG=1,NG
            SAVG(IEL,IG)=REAL(EVECT((IG-1)*LL4F+IEL))
          ENDDO
        ENDDO
        !
        ! compute nodal coefficients
        DO I=1,NX
          DELX=XXX(I+1)-XXX(I)
          DO J=1,NY
            DELY=YYY(J+1)-YYY(J)
            IEL=(J-1)*NX+I
            IBM=MAT(IEL)
            IF(IBM == 0) CYCLE
            WORK1(:NG)=DIFF(IBM,:NG)
            WORK2(:NG)=SIGR(IBM,:NG)
            WORK3(:NG,:NG)=SCAT(IBM,:NG,:NG)
            WORK4(:NG)=CHI(IBM,:NG)
            WORK5(:NG)=SIGF(IBM,:NG)
            KK1=IQFR(1,I,J)
            KK2=IQFR(2,I,J)
            KK3=IQFR(3,I,J)
            KK4=IQFR(4,I,J)
            XYZ(2:3)=XXX(I:I+1)-XXX(I)
            IF(KK1 == -2) THEN
              ! reflection BOUNDARY condition
              XYZ(1)=2.0*XYZ(2)-XYZ(3)
            ELSE IF(KK1 < 0) THEN
              ! zero/void/albedo BOUNDARY condition
              XYZ(1)=-99999.
            ELSE
              ! left neighbour
              XYZ(1)=XXX(I-1)-XXX(I)
            ENDIF
            IF(KK2 == -2) THEN
              ! reflection BOUNDARY condition
              XYZ(4)=2.0*XYZ(3)-XYZ(2)
            ELSE IF(KK2 < 0) THEN
              ! zero/void/albedo BOUNDARY condition
              XYZ(4)=-99999.
            ELSE
              ! right neighbour
              XYZ(4)=XXX(I+2)-XXX(I)
            ENDIF
            CALL NSSLR2(KEFF,NG,BNDTL,XYZ,DELY,WORK1,WORK2,WORK3,WORK4,
     >      WORK5,Lx(1,1,IEL),Rx(1,1,IEL))
            XYZ(2:3)=YYY(J:J+1)-YYY(J)
            IF(KK3 == -2) THEN
              ! reflection BOUNDARY condition
              XYZ(1)=2.0*XYZ(2)-XYZ(3)
            ELSE IF(KK3 < 0) THEN
              ! zero/void/albedo BOUNDARY condition
              XYZ(1)=-99999.
            ELSE
              ! left neighbour
              XYZ(1)=YYY(J-1)-YYY(J)
            ENDIF
            IF(KK4 == -2) THEN
              ! reflection BOUNDARY condition
              XYZ(4)=2.0*XYZ(3)-XYZ(2)
            ELSE IF(KK4 < 0) THEN
              ! zero/void/albedo BOUNDARY condition
              XYZ(4)=-99999.0
            ELSE
              ! right neighbour
              XYZ(4)=YYY(J+2)-YYY(J)
            ENDIF
            CALL NSSLR2(KEFF,NG,BNDTL,XYZ,DELX,WORK1,WORK2,WORK3,WORK4,
     >      WORK5,Ly(1,1,IEL),Ry(1,1,IEL))
          ENDDO
        ENDDO
        !
        ! one- and two-node anm relations
        CALL NSSANM2(NUN,NX,NY,LL4F,LL4X,NG,NMIX,IDL,KN,IQFR,QFR2,MAT,
     >  FD,Lx,Rx,Ly,Ry,SAVG)
        !
        ! compute drift coefficients
        DRIFT(:6,:NEL,:NG)=0.0
        DO IG=1,NG
          DO J=1,NY
            DO I=1,NX
              IEL=(J-1)*NX+I
              ! X- RELATIONS
              IND1=IDL(IEL)
              IF(IND1 == 0) CYCLE
              KK1=IQFR(1,I,J)
              KK2=IQFR(2,I,J)
              JXM=KN(1,I,J) ; JXP=KN(2,I,J)
              JYM=KN(3,I,J) ; JYP=KN(4,I,J)
              CALL NSSCO(NX,NY,NZ,NMIX,I,J,1,MAT,XX,YY,ZZ,DIFF(1,IG),
     >        IQFR(1,I,J),QFR2(1,IEL,IG),COEF)
              IF(KK1 < 0) THEN
                IND3=IDL((J-1)*NX+I+1)
                DRIFT(1,IEL,IG)=-(SAVG(5*LL4F+JXM,IG)+COEF(1)*
     >          SAVG(IND1,IG))/SAVG(IND1,IG)
                IF(IND3 /= 0) DRIFT(2,IEL,IG)=-(SAVG(5*LL4F+JXP,IG)+
     >          COEF(2)*(SAVG(IND3,IG)-SAVG(IND1,IG)))/(SAVG(IND3,IG)+
     >          SAVG(IND1,IG))
              ELSE IF(KK2 < 0) THEN
                IND2=IDL((J-1)*NX+I-1)
                IF(IND2 /= 0) DRIFT(1,IEL,IG)=-(SAVG(5*LL4F+JXM,IG)+
     >          COEF(1)*(SAVG(IND1,IG)-SAVG(IND2,IG)))/(SAVG(IND1,IG)+
     >          SAVG(IND2,IG))
                DRIFT(2,IEL,IG)=-(SAVG(5*LL4F+JXP,IG)-COEF(2)*
     >          SAVG(IND1,IG))/SAVG(IND1,IG)
              ELSE
                IND2=IDL((J-1)*NX+I-1)
                IND3=IDL((J-1)*NX+I+1)
                IF(IND2 /= 0) DRIFT(1,IEL,IG)=-(SAVG(5*LL4F+JXM,IG)+
     >          COEF(1)*(SAVG(IND1,IG)-SAVG(IND2,IG)))/(SAVG(IND1,IG)+
     >          SAVG(IND2,IG))
                IF(IND3 /= 0) DRIFT(2,IEL,IG)=-(SAVG(5*LL4F+JXP,IG)+
     >          COEF(2)*(SAVG(IND3,IG)-SAVG(IND1,IG)))/(SAVG(IND3,IG)+
     >          SAVG(IND1,IG))
              ENDIF
              KK3=IQFR(3,I,J)
              KK4=IQFR(4,I,J)
              IF(KK3 < 0) THEN
                IND3=IDL(J*NX+I)
                DRIFT(3,IEL,IG)=-(SAVG(5*LL4F+LL4X+JYM,IG)+COEF(3)*
     >          SAVG(IND1,IG))/SAVG(IND1,IG)
                IF(IND3 /= 0) DRIFT(4,IEL,IG)=-(SAVG(5*LL4F+LL4X+JYP,IG)
     >          +COEF(4)*(SAVG(IND3,IG)-SAVG(IND1,IG)))/(SAVG(IND3,IG)+
     >          SAVG(IND1,IG))
              ELSE IF(KK4 < 0) THEN
                IND2=IDL((J-2)*NX+I)
                IF(IND2 /= 0) DRIFT(3,IEL,IG)=-(SAVG(5*LL4F+LL4X+JYM,IG)
     >          +COEF(3)*(SAVG(IND1,IG)-SAVG(IND2,IG)))/(SAVG(IND1,IG)+
     >          SAVG(IND2,IG))
                DRIFT(4,IEL,IG)=-(SAVG(5*LL4F+LL4X+JYP,IG)-COEF(4)*
     >          SAVG(IND1,IG))/SAVG(IND1,IG)
              ELSE
                IND2=IDL((J-2)*NX+I)
                IND3=IDL(J*NX+I)
                IF(IND2 /= 0) DRIFT(3,IEL,IG)=-(SAVG(5*LL4F+LL4X+JYM,IG)
     >          +COEF(3)*(SAVG(IND1,IG)-SAVG(IND2,IG)))/(SAVG(IND1,IG)+
     >          SAVG(IND2,IG))
                IF(IND3 /= 0) DRIFT(4,IEL,IG)=-(SAVG(5*LL4F+LL4X+JYP,IG)
     >          +COEF(4)*(SAVG(IND3,IG)-SAVG(IND1,IG)))/(SAVG(IND3,IG)+
     >          SAVG(IND1,IG))
              ENDIF
            ENDDO
          ENDDO
        ENDDO
        IF(IMPX > 5) THEN
          DO IG=1,NG
            WRITE(6,'(28H NSSFL4: DRIFT COEFFICIENTS(,I5,2H):)') IG
            DO IEL=1,NX*NY
              WRITE(6,'(1P,I7,4E12.4)') IEL,DRIFT(:4,IEL,IG)
            ENDDO
          ENDDO
        ENDIF
      ENDDO
*----
*  END OF NODAL CORRECTION LOOP
*----
      IF(IMPX.GT.0) WRITE(6,20) KEFF,JTER
*----
*  COMPUTE BOUNDARY FLUXES
*----
      ! X-directed boundary fluxes
      DO J=1,NY
        DO I=1,NX
          ! X- RELATIONS
          IEL=(J-1)*NX+I
          IND=IDL(IEL)
          IF(IND == 0) CYCLE
          JXM=KN(1,I,J) ; JXP=KN(2,I,J) ; JYM=KN(3,I,J) ; JYP=KN(4,I,J)
          JYM_M=0 ; JYP_M=0 ; JYM_P=0 ; JYP_P=0
          IF((I == 1).AND.(IQFR(1,1,J) == -2)) THEN
            JYM_M=KN(3,1,J) ; JYP_M=KN(4,1,J)
          ELSE IF(I > 1) THEN
            JYM_M=KN(3,I-1,J) ; JYP_M=KN(4,I-1,J)
          ENDIF
          IF((I == NX).AND.(IQFR(2,NX,J) == -2)) THEN
            JYM_P=KN(3,NX,J) ; JYP_P=KN(4,NX,J)
          ELSE IF(I < NX) THEN
            JYM_P=KN(3,I+1,J) ; JYP_P=KN(4,I+1,J)
          ENDIF
          SAVG(LL4F+IND,:NG)=REAL(MATMUL(Lx(:NG,:NG,IEL),
     >    SAVG(IND,:NG)),4)
          IF(JXM /= 0) SAVG(LL4F+IND,:NG)=REAL(SAVG(LL4F+IND,:NG)+
     >    MATMUL(Lx(:NG,NG+1:2*NG,IEL),SAVG(5*LL4F+JXM,:NG)),4)
          IF(JYM_M /= 0) SAVG(LL4F+IND,:NG)=REAL(SAVG(LL4F+IND,:NG)+
     >    MATMUL(Lx(:NG,2*NG+1:3*NG,IEL),SAVG(5*LL4F+LL4X+JYM_M,:NG)),4)
          IF(JYM /= 0) SAVG(LL4F+IND,:NG)=REAL(SAVG(LL4F+IND,:NG)+
     >    MATMUL(Lx(:NG,3*NG+1:4*NG,IEL),SAVG(5*LL4F+LL4X+JYM,:NG)),4)
          IF(JYM_P /= 0) SAVG(LL4F+IND,:NG)=REAL(SAVG(LL4F+IND,:NG)+
     >    MATMUL(Lx(:NG,4*NG+1:5*NG,IEL),SAVG(5*LL4F+LL4X+JYM_P,:NG)),4)
          IF(JYP_M /= 0) SAVG(LL4F+IND,:NG)=REAL(SAVG(LL4F+IND,:NG)+
     >    MATMUL(Lx(:NG,5*NG+1:6*NG,IEL),SAVG(5*LL4F+LL4X+JYP_M,:NG)),4)
          IF(JYP /= 0) SAVG(LL4F+IND,:NG)=REAL(SAVG(LL4F+IND,:NG)+
     >    MATMUL(Lx(:NG,6*NG+1:7*NG,IEL),SAVG(5*LL4F+LL4X+JYP,:NG)),4)
          IF(JYP_P /= 0) SAVG(LL4F+IND,:NG)=REAL(SAVG(LL4F+IND,:NG)+
     >    MATMUL(Lx(:NG,7*NG+1:8*NG,IEL),SAVG(5*LL4F+LL4X+JYP_P,:NG)),4)
          !
          ! X+ RELATIONS
          SAVG(2*LL4F+IND,:NG)=REAL(MATMUL(Rx(:NG,:NG,IEL),
     >    SAVG(IND,:NG)),4)
          IF(JXP /= 0) SAVG(2*LL4F+IND,:NG)=REAL(SAVG(2*LL4F+IND,:NG)+
     >    MATMUL(Rx(:NG,NG+1:2*NG,IEL),SAVG(5*LL4F+JXP,:NG)),4)
          IF(JYM_M /= 0) SAVG(2*LL4F+IND,:NG)=REAL(SAVG(2*LL4F+IND,:NG)+
     >    MATMUL(Rx(:NG,2*NG+1:3*NG,IEL),SAVG(5*LL4F+LL4X+JYM_M,:NG)),4)
          IF(JYM /= 0) SAVG(2*LL4F+IND,:NG)=REAL(SAVG(2*LL4F+IND,:NG)+
     >    MATMUL(Rx(:NG,3*NG+1:4*NG,IEL),SAVG(5*LL4F+LL4X+JYM,:NG)),4)
          IF(JYM_P /= 0) SAVG(2*LL4F+IND,:NG)=REAL(SAVG(2*LL4F+IND,:NG)+
     >    MATMUL(Rx(:NG,4*NG+1:5*NG,IEL),SAVG(5*LL4F+LL4X+JYM_P,:NG)),4)
          IF(JYP_M /= 0) SAVG(2*LL4F+IND,:NG)=REAL(SAVG(2*LL4F+IND,:NG)+
     >    MATMUL(Rx(:NG,5*NG+1:6*NG,IEL),SAVG(5*LL4F+LL4X+JYP_M,:NG)),4)
          IF(JYP /= 0) SAVG(2*LL4F+IND,:NG)=REAL(SAVG(2*LL4F+IND,:NG)+
     >    MATMUL(Rx(:NG,6*NG+1:7*NG,IEL),SAVG(5*LL4F+LL4X+JYP,:NG)),4)
          IF(JYP_P /= 0) SAVG(2*LL4F+IND,:NG)=REAL(SAVG(2*LL4F+IND,:NG)+
     >    MATMUL(Rx(:NG,7*NG+1:8*NG,IEL),SAVG(5*LL4F+LL4X+JYP_P,:NG)),4)
        ENDDO
      ENDDO
      ! Y-directed boundary fluxes
      DO I=1,NX
        DO J=1,NY
          ! Y- RELATIONS
          IEL=(J-1)*NX+I
          IND=IDL(IEL)
          IF(IND == 0) CYCLE
          JXM_M=0 ; JXP_M=0 ; JXM_P=0 ; JXP_P=0
          JXM=KN(1,I,J) ; JXP=KN(2,I,J) ; JYM=KN(3,I,J) ; JYP=KN(4,I,J)
          IF((J == 1).AND.(IQFR(3,I,1) == -2)) THEN
            JXM_M=KN(1,I,1) ; JXP_M=KN(2,I,1)
          ELSE IF(J > 1) THEN
            JXM_M=KN(1,I,J-1) ; JXP_M=KN(2,I,J-1)
          ENDIF
          IF((J == NY).AND.(IQFR(4,I,NY) == -2)) THEN
            JXM_P=KN(1,I,NY) ; JXP_P=KN(2,I,NY)
          ELSE IF(J < NY) THEN
            JXM_P=KN(1,I,J+1) ; JXP_P=KN(2,I,J+1)
          ENDIF
          SAVG(3*LL4F+IND,:NG)=REAL(MATMUL(Ly(:NG,:NG,IEL),
     >    SAVG(IND,:NG)),4)
          IF(JYM /= 0) SAVG(3*LL4F+IND,:NG)=REAL(SAVG(3*LL4F+IND,:NG)+
     >    MATMUL(Ly(:NG,NG+1:2*NG,IEL),SAVG(5*LL4F+LL4X+JYM,:NG)),4)
          IF(JXM_M /= 0) SAVG(3*LL4F+IND,:NG)=REAL(SAVG(3*LL4F+IND,:NG)+
     >    MATMUL(Ly(:NG,2*NG+1:3*NG,IEL),SAVG(5*LL4F+JXM_M,:NG)),4)
          IF(JXM /= 0) SAVG(3*LL4F+IND,:NG)=REAL(SAVG(3*LL4F+IND,:NG)+
     >    MATMUL(Ly(:NG,3*NG+1:4*NG,IEL),SAVG(5*LL4F+JXM,:NG)),4)
          IF(JXM_P /= 0) SAVG(3*LL4F+IND,:NG)=REAL(SAVG(3*LL4F+IND,:NG)+
     >    MATMUL(Ly(:NG,4*NG+1:5*NG,IEL),SAVG(5*LL4F+JXM_P,:NG)),4)
          IF(JXP_M /= 0) SAVG(3*LL4F+IND,:NG)=REAL(SAVG(3*LL4F+IND,:NG)+
     >    MATMUL(Ly(:NG,5*NG+1:6*NG,IEL),SAVG(5*LL4F+JXP_M,:NG)),4)
          IF(JXP /= 0) SAVG(3*LL4F+IND,:NG)=REAL(SAVG(3*LL4F+IND,:NG)+
     >    MATMUL(Ly(:NG,6*NG+1:7*NG,IEL),SAVG(5*LL4F+JXP,:NG)),4)
          IF(JXP_P /= 0) SAVG(3*LL4F+IND,:NG)=REAL(SAVG(3*LL4F+IND,:NG)+
     >    MATMUL(Ly(:NG,7*NG+1:8*NG,IEL),SAVG(5*LL4F+JXP_P,:NG)),4)
          !
          ! Y+ RELATIONS
          SAVG(4*LL4F+IND,:NG)=REAL(MATMUL(Ry(:NG,:NG,IEL),
     >    SAVG(IND,:NG)),4)
          IF(JYP /= 0) SAVG(4*LL4F+IND,:NG)=REAL(SAVG(4*LL4F+IND,:NG)+
     >    MATMUL(Ry(:NG,NG+1:2*NG,IEL),SAVG(5*LL4F+LL4X+JYP,:NG)),4)
          IF(JXM_M /= 0) SAVG(4*LL4F+IND,:NG)=REAL(SAVG(4*LL4F+IND,:NG)+
     >    MATMUL(Ry(:NG,2*NG+1:3*NG,IEL),SAVG(5*LL4F+JXM_M,:NG)),4)
          IF(JXM /= 0) SAVG(4*LL4F+IND,:NG)=REAL(SAVG(4*LL4F+IND,:NG)+
     >    MATMUL(Ry(:NG,3*NG+1:4*NG,IEL),SAVG(5*LL4F+JXM,:NG)),4)
          IF(JXM_P /= 0) SAVG(4*LL4F+IND,:NG)=REAL(SAVG(4*LL4F+IND,:NG)+
     >    MATMUL(Ry(:NG,4*NG+1:5*NG,IEL),SAVG(5*LL4F+JXM_P,:NG)),4)
          IF(JXP_M /= 0) SAVG(4*LL4F+IND,:NG)=REAL(SAVG(4*LL4F+IND,:NG)+
     >    MATMUL(Ry(:NG,5*NG+1:6*NG,IEL),SAVG(5*LL4F+JXP_M,:NG)),4)
          IF(JXP /= 0) SAVG(4*LL4F+IND,:NG)=REAL(SAVG(4*LL4F+IND,:NG)+
     >    MATMUL(Ry(:NG,6*NG+1:7*NG,IEL),SAVG(5*LL4F+JXP,:NG)),4)
          IF(JXP_P /= 0) SAVG(4*LL4F+IND,:NG)=REAL(SAVG(4*LL4F+IND,:NG)+
     >    MATMUL(Ry(:NG,7*NG+1:8*NG,IEL),SAVG(5*LL4F+JXP_P,:NG)),4)
        ENDDO
      ENDDO
      IF(IMPX > 1) THEN
        WRITE(6,'(/21H NSSFL4: UNKNOWNS----)')
        DO IG=1,NG
          WRITE(6,'(14H NSSFL4: SAVG(,I4,2H)=)') IG
          WRITE(6,'(1P,12E12.4)') SAVG(:LL4F,IG)
          WRITE(6,'(19H X-BOUNDARY FLUXES:)')
          WRITE(6,'(1P,12E12.4)') SAVG(LL4F+1:2*LL4F,IG)
          WRITE(6,'(1P,12E12.4)') SAVG(2*LL4F+1:3*LL4F,IG)
          WRITE(6,'(19H Y-BOUNDARY FLUXES:)')
          WRITE(6,'(1P,12E12.4)') SAVG(3*LL4F+1:4*LL4F,IG)
          WRITE(6,'(1P,12E12.4)') SAVG(4*LL4F+1:5*LL4F,IG)
          WRITE(6,'(12H X-CURRENTS:)')
          WRITE(6,'(1P,12E12.4)') SAVG(5*LL4F+1:5*LL4F+LL4X,IG)
          WRITE(6,'(12H Y-CURRENTS:)')
          WRITE(6,'(1P,12E12.4)') SAVG(5*LL4F+LL4X+1:NUN,IG)
          WRITE(6,'(5H ----)')
        ENDDO
      ENDIF
*----
*  SCRATCH STORAGE DEALLOCATION
*----
      DEALLOCATE(EVECT,A,AGGXY)
      DEALLOCATE(WORK5,WORK4,WORK3,WORK2,WORK1,DRIFT)
      DEALLOCATE(Ry,Ly,Rx,Lx)
      DEALLOCATE(ZZ,QFR2)
*
   10 FORMAT(14H NSSFL4: JTER=,I4,11H CMFD KEFF=,1P E13.6,
     1 12H OBTAINED IN,I4,12H ITERATIONS.)
   20 FORMAT(18H NSSFL4: ANM KEFF=,F11.8,12H OBTAINED IN,I5,
     1 11H ITERATIONS)
      END
