subroutine NSSMXYZ(ll4f,ndim,nx,ny,nz,nmix,mat,xx,yy,zz,idl,vol,iqfr,qfr,diff,drift,sigt,aggxyz)
!
!-----------------------------------------------------------------------
!
!Purpose:
! Assembly of system matrices for coarse mesh finite differences with
! nodal correction in 2D.
!
!Copyright:
! Copyright (C) 2022 Ecole Polytechnique de Montreal
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version
!
!Author(s): A. Hebert
!
!Parameters: input
! ll4f    number of unknowns per energy group.
! ndim    number of dimensions (1, 2, or 3).
! nx      number of nodes in the X direction.
! ny      number of nodes in the Y direction.
! nz      number of nodes in the Z direction.
! nmix    number of mixtures.
! mat     node mixtures.
! xx      node widths in the X direction.
! yy      node widths in the Y direction.
! zz      node widths in the Z direction.
! idl     position of averaged fluxes in unknown vector.
! vol     node volumes.
! iqfr    boundary conditions.
! qfr     albedo functions.
! diff    diffusion coefficients.
! drift   drift coefficients.
! sigt    removal macroscopic cross section.
!
!Parameters: output
! aggxyz  matrix corresponding to the divergence (i.e. leakage) and
!         removal terms.
!
!-----------------------------------------------------------------------
!
  !----
  !  SUBROUTINE ARGUMENTS
  !----
  integer :: ll4f,ndim,nx,ny,nz,nmix,mat(nx,ny,nz),idl(nx,ny,nz),iqfr(6,nx,ny,nz)
  real :: xx(nx,ny,nz),yy(nx,ny,nz),zz(nx,ny,nz),vol(nx,ny,nz),qfr(6,nx,ny,nz), &
  & diff(nmix),drift(6,nx,ny,nz),sigt(nmix),aggxyz(ll4f,ll4f)
  real :: coef(6),codr(6)
  !
  aggxyz(:ll4f,:ll4f)=0.0
  do k=1,nz
    do j=1,ny
      do i=1,nx
        ibm=mat(i,j,k)
        if(ibm <= 0) cycle
        ind1=idl(i,j,k)
        if(ind1 == 0) cycle
        vol0=vol(i,j,k)
        call NSSCO(nx,ny,nz,nmix,i,j,k,mat,xx,yy,zz,diff,iqfr(1,i,j,k),qfr(1,i,j,k),coef)
        coef(1:2)=coef(1:2)*vol0/xx(i,j,k)
        coef(3:4)=coef(3:4)*vol0/yy(i,j,k)
        coef(5:6)=coef(5:6)*vol0/zz(i,j,k)
        !
        ! x-directed couplings
        codr(1)=drift(1,i,j,k)*vol0/xx(i,j,k)
        codr(2)=drift(2,i,j,k)*vol0/xx(i,j,k)
        ind2=0
        kk1=iqfr(1,i,j,k)
        if(kk1 == -4) then
          ind2=idl(nx,j,k)
        else if(kk1 == 0) then
          ind2=idl(i-1,j,k)
        endif
        if(ind2 /= 0) aggxyz(ind1,ind2)=aggxyz(ind1,ind2)-coef(1)+codr(1)
        ind2=0
        kk2=iqfr(2,i,j,k)
        if(kk2 == -4) then
          ind2=idl(1,j,k)
        else if(kk2 == 0) then
          ind2=idl(i+1,j,k)
        endif
        if(ind2 /= 0) aggxyz(ind1,ind2)=aggxyz(ind1,ind2)-coef(2)-codr(2)
        aggxyz(ind1,ind1)=aggxyz(ind1,ind1)+coef(1)+codr(1)+coef(2)-codr(2)
        !
        if(ndim > 1) then
          ! y-directed couplings
          codr(3)=drift(3,i,j,k)*vol0/yy(i,j,k)
          codr(4)=drift(4,i,j,k)*vol0/yy(i,j,k)
          ind2=0
          kk3=iqfr(3,i,j,k)
          if(kk3 == -4) then
            ind2=idl(i,ny,k)
          else if(kk3 == 0) then
            ind2=idl(i,j-1,k)
          endif
          if(ind2 /= 0) aggxyz(ind1,ind2)=aggxyz(ind1,ind2)-coef(3)+codr(3)
          ind2=0
          kk4=iqfr(4,i,j,k)
          if(kk4 == -4) then
            ind2=idl(i,1,k)
          else if(kk4 == 0) then
            ind2=idl(i,j+1,k)
          endif
          if(ind2 /= 0) aggxyz(ind1,ind2)=aggxyz(ind1,ind2)-coef(4)-codr(4)
          aggxyz(ind1,ind1)=aggxyz(ind1,ind1)+coef(3)+codr(3)+coef(4)-codr(4)
        endif
        !
        if(ndim > 2) then
          ! z-directed couplings
          codr(5)=drift(5,i,j,k)*vol0/zz(i,j,k)
          codr(6)=drift(6,i,j,k)*vol0/zz(i,j,k)
          ind2=0
          kk5=iqfr(5,i,j,k)
          if(kk5 == -4) then
            ind2=idl(i,j,nz)
          else if(kk5 == 0) then
            ind2=idl(i,j,k-1)
          endif
          if(ind2 /= 0) aggxyz(ind1,ind2)=aggxyz(ind1,ind2)-coef(5)+codr(5)
          ind2=0
          kk6=iqfr(6,i,j,k)
          if(kk6 == -4) then
            ind2=idl(i,j,1)
          else if(kk6 == 0) then
            ind2=idl(i,j,k+1)
          endif
          if(ind2 /= 0) aggxyz(ind1,ind2)=aggxyz(ind1,ind2)-coef(6)-codr(6)
          aggxyz(ind1,ind1)=aggxyz(ind1,ind1)+coef(5)+codr(5)+coef(6)-codr(6)
        endif
        !
        aggxyz(ind1,ind1)=aggxyz(ind1,ind1)+sigt(ibm)*vol0
      enddo
    enddo
  enddo
end subroutine NSSMXYZ
